@@ -35,7 +35,7 @@ class Work:
             ],
         depends=['parent', 'id'])
     leveling_delay = fields.Float("Leveling Delay")
-    back_leveling_delay = fields.Float("Leveling Delay")
+    back_leveling_delay = fields.Float("Back Leveling Delay")
     allocations = fields.One2Many('project.allocation', 'work', 'Allocations',
         states={
             'invisible': Eval('type') != 'task',
@@ -73,13 +73,11 @@ class Work:
     @classmethod
     def __setup__(cls):
         super(Work, cls).__setup__()
-        cls._constraints += [
-            ('check_recursion', 'recursive_dependency'),
-            ]
-        cls._error_messages.update({
-                'recursive_dependency': 'You can not create recursive '
-                        'dependencies!',
-                })
+
+    @classmethod
+    def validate(cls, works):
+        super(Work, cls).validate(works)
+        cls.check_recursion(works)
 
     @classmethod
     def check_recursion(cls, records, parent='successors'):
@@ -131,12 +129,11 @@ class Work:
 
                 total_allocation = 0
                 if not work.allocations:
-                    durations[work.id] = (work.effort or 0)
+                    durations[work.id] = work.effort
                     continue
                 for allocation in work.allocations:
                     total_allocation += allocation.percentage
-                durations[work.id] = ((work.effort or 0)
-                    / (total_allocation / 100.0))
+                durations[work.id] = work.effort / (total_allocation / 100.0)
 
             while leafs:
                 for work_id in leafs:
@@ -192,7 +189,8 @@ class Work:
 
                 target_ids = len(v) > 1 and v[1] or []
                 if operator == 'create':
-                    to_link.append(Request.create(v[1]).id)
+                    request, = Request.create(v[1])
+                    to_link.append(request.id)
                 elif operator == 'write':
                     Request.write([Request(v[1])], v[2])
                 elif operator == 'delete':
@@ -228,12 +226,15 @@ class Work:
                             req_refs.append(ref)
                 RequestReference.delete(req_refs)
 
+                to_create = []
                 for i in to_link:
                     for work in works:
-                        RequestReference.create({
+                        to_create.append({
                                 'request': i,
                                 'reference': str(work),
                                 })
+                if to_create:
+                    RequestReference.create(to_create)
             return
 
         fun_fields = ('actual_start_date', 'actual_finish_date',
@@ -243,10 +244,10 @@ class Work:
         for fun_field, db_field in zip(fun_fields, db_fields):
             if fun_field == name:
                 cls.write(works, {
-                        db_field: value \
-                                and datetime.datetime.combine(value,
-                                    datetime.time()) \
-                                or None,
+                        db_field: (value
+                            and datetime.datetime.combine(value,
+                                datetime.time())
+                            or None),
                         })
                 break
 
@@ -345,7 +346,7 @@ class Work:
         mindate = lambda x, y: x and y and min(x, y) or x or y
 
         # propagate constraint_start_time
-        constraint_start = reduce(maxdate, (pred.early_finish_time \
+        constraint_start = reduce(maxdate, (pred.early_finish_time
                 for pred in self.predecessors), None)
 
         if constraint_start is None and self.parent:
@@ -426,7 +427,7 @@ class Work:
                 parent = work
 
         # propagate constraint_finish_time
-        constraint_finish = reduce(mindate, (succ.late_start_time \
+        constraint_finish = reduce(mindate, (succ.late_start_time
                 for succ in self.successors), None)
 
         if constraint_finish is None and self.parent:
@@ -597,8 +598,9 @@ class Work:
 
                 yield sibling, delay
 
+        parent = self.parent and self.parent.id or None
         siblings = self.search([
-                ('parent', '=', self.parent.id if self.parent else None)
+                ('parent', '=', parent.id)
                 ])
 
         refkey = get_key(self)
@@ -615,8 +617,8 @@ class Work:
                     'back_leveling_delay': delay,
                     })
 
-        if self.parent:
-            self.parent.compute_dates()
+        if parent:
+            parent.compute_dates()
 
     @classmethod
     def write(cls, works, values):
@@ -632,11 +634,12 @@ class Work:
                 work.compute_dates()
 
     @classmethod
-    def create(cls, values):
-        work = super(Work, cls).create(values)
-        work.reset_leveling()
-        work.compute_dates()
-        return work
+    def create(cls, vlist):
+        works = super(Work, cls).create(vlist)
+        for work in works:
+            work.reset_leveling()
+            work.compute_dates()
+        return works
 
     @classmethod
     def delete(cls, works):
@@ -692,15 +695,16 @@ class PredecessorSuccessor(ModelSQL):
             parent.compute_dates()
 
     @classmethod
-    def create(cls, values):
-        pred_succ = super(PredecessorSuccessor, cls).create(values)
+    def create(cls, vlist):
+        pred_succs = super(PredecessorSuccessor, cls).create(vlist)
 
-        pred_succ.predecessor.reset_leveling()
-        pred_succ.successor.reset_leveling()
+        for pred_succ in pred_succs:
+            pred_succ.predecessor.reset_leveling()
+            pred_succ.successor.reset_leveling()
 
-        if pred_succ.predecessor.parent:
-            pred_succ.predecessor.parent.compute_dates()
-        return pred_succ
+            if pred_succ.predecessor.parent:
+                pred_succ.predecessor.parent.compute_dates()
+        return pred_succs
 
 
 class Leveling(Wizard):
